#!/usr/bin/env python
#
# test_importall.py -
#
# Author: Paul McCarthy <pauldmccarthy@gmail.com>
#


import pkgutil


def test_importall():
    import fsleyes_props as props

    for _, module, _ in pkgutil.iter_modules(props.__path__, 'fsleyes_props.'):
        __import__(module)



#!/usr/bin/env python
#
# widgets_point.py - Create widgets for modifying Point properties.
#
# Author: Paul McCarthy <pauldmccarthy@gmail.com>
#
"""This module provides the :func:`_Point` function, which is imported into
the :mod:`widgets` module namespace. It is separated purely to keep the
``widgets`` module file size down.
"""


import wx

import fsleyes_widgets.floatslider as floatslider
from . import                         widgets


def _Point(parent,
           hasProps,
           propObj,
           propVal,
           labels=None,
           showLimits=True,
           editLimits=False,
           mousewheel=False):
    """Creates and returns a :class:`.SliderSpinPanel` allowing the user to
    edit the low/high values along each dimension of the given
    :class:`.Point` property value.

    :arg labels:     One label for each dimension, to be shown alongside the
                     corresponding controls.

    :arg showLimits: Show labels displaying the point limits.

    :arg editLimits: Show buttons allowing the user to edit the point limits.

    :arg mousewheel: The user can use the mouse wheel to change the point
                     value.

    See the :func:`.widgets._String` documentation for details on the other
    parameters.
    """
    panel = wx.Panel(parent)
    sizer = wx.BoxSizer(wx.VERTICAL)
    panel.SetSizer(sizer)

    ndims  = propObj._ndims
    real   = propObj._real

    if labels is None:
        labels = [None] * ndims

    for dim in range(len(propVal)):

        style = 0
        if not real:   style |= floatslider.SSP_INTEGER
        if showLimits: style |= floatslider.SSP_SHOW_LIMITS
        if editLimits: style |= floatslider.SSP_EDIT_LIMITS
        if mousewheel: style |= floatslider.SSP_MOUSEWHEEL

        slider = floatslider.SliderSpinPanel(
            panel,
            value=propVal[dim],
            minValue=propVal.getMin(dim),
            maxValue=propVal.getMax(dim),
            label=labels[dim],
            style=style)

        sizer.Add(slider, flag=wx.EXPAND)

        _pointBind(hasProps, propObj, propVal, slider, dim)

    panel.Layout()

    return panel


def _pointBind(hasProps, propObj, propVal, slider, dim, editLimits):
    """Called by the :func:`_Point` function.

    Binds the given :class:`.SliderSpinPanel` to one dimension of the given
    :class:`.PointValueList` so that changes in one are propagated to the
    other.

    :arg slider: The :class:`.SliderSpinPanel` instance.

    :arg dim:    The 0-indexed dimension of the :class:`.Point` value.

    See :func:`_Point` for details on the other arguments.
    """

    dimPropVal = propVal.getPropertyValueList()[dim]

    widgets._propBind(hasProps,
                      propObj._listType,
                      dimPropVal,
                      slider,
                      floatslider.EVT_SSP_VALUE)

    def propLimitsChanged(*a):
        minval = propVal.getMin(dim)
        maxval = propVal.getMax(dim)

        if minval is not None: slider.SetMin(minval)
        if maxval is not None: slider.SetMax(maxval)

    def sliderLimitsChanged(ev):
        propVal.setMin(dim, ev.min)
        propVal.setMax(dim, ev.max)
        ev.Skip()

    if editLimits:
        slider.Bind(floatslider.EVT_SSP_LIMIT, sliderLimitsChanged)

    lName = 'PointLimits_{}_{}'.format(id(slider), dim)

    dimPropVal.addAttributeListener(lName, propLimitsChanged, weak=False)

    def onDestroy(ev):
        dimPropVal.removeAttributeListener(lName)
        ev.Skip()

    slider.Bind(wx.EVT_WINDOW_DESTROY, onDestroy)



#!/usr/bin/env python
#
# test_property_number.py -
#
# Author: Paul McCarthy <pauldmccarthy@gmail.com>
#


import pytest

import fsleyes_props as props


def test_Int():

    class MyObj(props.HasProperties):
        unbounded           = props.Int()
        unbounded_default   = props.Int(default=10)
        bounded             = props.Int(minval=0, maxval=10)
        bounded_min         = props.Int(minval=0)
        bounded_max         = props.Int(maxval=10)
        bounded_clamped     = props.Int(clamped=True, minval=0, maxval=10)
        bounded_min_clamped = props.Int(clamped=True, minval=0)
        bounded_max_clamped = props.Int(clamped=True, maxval=10)


    obj = MyObj()

    # property, value, expected

    assert obj.unbounded_default == 10

    with pytest.raises(ValueError): obj.unbounded = ''
    with pytest.raises(ValueError): obj.unbounded = 'abcde'
    with pytest.raises(TypeError):  obj.unbounded = None

    testcases = [
        ('unbounded',         '-999', -999),
        ('unbounded',            '0',    0),
        ('unbounded',          '999',  999),
        ('unbounded',           -999, -999),
        ('unbounded',              0,    0),
        ('unbounded',            999,  999),
        ('unbounded',           -1.7,   -1),
        ('unbounded',           -1.2,   -1),
        ('unbounded',           -0.7,    0),
        ('unbounded',           -0.3,    0),
        ('unbounded',            0.3,    0),
        ('unbounded',            0.7,    0),
        ('unbounded',            1.2,    1),
        ('unbounded',            1.7,    1),
        ('bounded',             -999, -999),
        ('bounded',                0,    0),
        ('bounded',              999,  999),
        ('bounded_min',         -999, -999),
        ('bounded_min',            0,    0),
        ('bounded_min',          999,  999),
        ('bounded_max',         -999, -999),
        ('bounded_max',            0,    0),
        ('bounded_max',          999,  999),
        ('bounded_clamped',     -999,    0),
        ('bounded_clamped',       -1,    0),
        ('bounded_clamped',        0,    0),
        ('bounded_clamped',        5,    5),
        ('bounded_clamped',       10,   10),
        ('bounded_clamped',       11,   10),
        ('bounded_clamped',      999,   10),
        ('bounded_min_clamped', -999,    0),
        ('bounded_min_clamped',   -1,    0),
        ('bounded_min_clamped',    0,    0),
        ('bounded_min_clamped',    5,    5),
        ('bounded_min_clamped',   10,   10),
        ('bounded_min_clamped',  999,  999),
        ('bounded_max_clamped', -999, -999),
        ('bounded_max_clamped',   -1,   -1),
        ('bounded_max_clamped',    0,    0),
        ('bounded_max_clamped',    5,    5),
        ('bounded_max_clamped',   10,   10),
        ('bounded_max_clamped',   11,   10),
        ('bounded_max_clamped',  999,   10),
    ]


    for prop, val, exp in testcases:

        setattr(obj, prop, val)
        assert getattr(obj, prop) == exp

def test_Real():

    class MyObj(props.HasProperties):
        unbounded           = props.Real()
        unbounded_default   = props.Real(default=10)
        bounded             = props.Real(minval=0, maxval=10)
        bounded_min         = props.Real(minval=0)
        bounded_max         = props.Real(maxval=10)
        bounded_clamped     = props.Real(clamped=True, minval=0, maxval=10)
        bounded_min_clamped = props.Real(clamped=True, minval=0)
        bounded_max_clamped = props.Real(clamped=True, maxval=10)


    obj = MyObj()

    assert obj.unbounded_default == 10
    assert obj.bounded_min       == 0
    assert obj.bounded_max       == 10
    assert obj.bounded           == 5

    # property, value, expected
    testcases = [
        ('unbounded',         '-0.5', -0.5),
        ('unbounded',          '0.0',  0.0),
        ('unbounded',          '0.5',  0.5),
        ('unbounded',         '-999', -999),
        ('unbounded',            '0',    0),
        ('unbounded',          '999',  999),
        ('unbounded',           -999, -999),
        ('unbounded',              0,    0),
        ('unbounded',            999,  999),
        ('bounded',             -999, -999),
        ('bounded',                0,    0),
        ('bounded',              999,  999),
        ('bounded_min',         -999, -999),
        ('bounded_min',            0,    0),
        ('bounded_min',          999,  999),
        ('bounded_max',         -999, -999),
        ('bounded_max',            0,    0),
        ('bounded_max',          999,  999),
        ('bounded_clamped',     -999,    0),
        ('bounded_clamped',       -1,    0),
        ('bounded_clamped',        0,    0),
        ('bounded_clamped',        5,    5),
        ('bounded_clamped',       10,   10),
        ('bounded_clamped',       11,   10),
        ('bounded_clamped',      999,   10),
        ('bounded_min_clamped', -999,    0),
        ('bounded_min_clamped',   -1,    0),
        ('bounded_min_clamped',    0,    0),
        ('bounded_min_clamped',    5,    5),
        ('bounded_min_clamped',   10,   10),
        ('bounded_min_clamped',  999,  999),
        ('bounded_max_clamped', -999, -999),
        ('bounded_max_clamped',   -1,   -1),
        ('bounded_max_clamped',    0,    0),
        ('bounded_max_clamped',    5,    5),
        ('bounded_max_clamped',   10,   10),
        ('bounded_max_clamped',   11,   10),
        ('bounded_max_clamped',  999,   10),
    ]


    for prop, val, exp in testcases:

        setattr(obj, prop, val)
        assert getattr(obj, prop) == exp


def test_Percentage():

    class MyObj(props.HasProperties):
        default                 = props.Percentage()
        default_clamped         = props.Percentage(clamped=True)
        default_bounded_clamped = props.Percentage(clamped=True,
                                                   minval=100,
                                                   maxval=1000)

    obj = MyObj()

    testcases = [
        ('default',                  -500,   -500),
        ('default',                  -100,   -100),
        ('default',                     0,      0),
        ('default',                   100,    100),
        ('default',                   500,    500),
        ('default_clamped',          -500,      0),
        ('default_clamped',          -100,      0),
        ('default_clamped',             0,      0),
        ('default_clamped',           100,    100),
        ('default_clamped',           500,    100),

        ('default_bounded_clamped',  -500,    100),
        ('default_bounded_clamped',  -100,    100),
        ('default_bounded_clamped',     0,    100),
        ('default_bounded_clamped',   100,    100),
        ('default_bounded_clamped',   500,    500),
        ('default_bounded_clamped',  1000,   1000),
        ('default_bounded_clamped',  1001,   1000),
    ]

    for prop, val, exp in testcases:

        setattr(obj, prop, val)
        assert getattr(obj, prop) == exp
